﻿using System;
using System.Collections.Generic;
using SharpDX;

namespace PackingPlanVisualizations {

  public class CenteredContainer {
    private  BasicCuboidShape container;
    private  List<BasicCuboidShape> packingItems;
    private float normalizingFactor;

    public CenteredContainer(BasicCuboidShape container) {
      normalizingFactor = CalculateNormalizingFactor(container);
      this.container = Normalize (container);
      packingItems = new List<BasicCuboidShape>();
    }

    private float CalculateNormalizingFactor(BasicCuboidShape shape) {
      float longestEdge = Math.Max(Math.Max(shape.ShapeSize.X, shape.ShapeSize.Y), shape.ShapeSize.Z);
      return 1 / (longestEdge / 30);
    }
    private BasicCuboidShape Normalize(BasicCuboidShape shape) {
      return new BasicCuboidShape(shape.ShapeSize * normalizingFactor, shape.ShapePosition * normalizingFactor, shape.ShapeID, shape.Material);
    }



    public BasicCuboidShape Container {
      get { return container; }
    }
    public List<BasicCuboidShape> PackingItems {
      get { return packingItems; }
    }


    public void AddPackingItem(Vector3 size, Vector3 position, int index, int material) {
      Vector3 itemSize = size * normalizingFactor;
      Vector3 itemPosition = position * normalizingFactor;
      //Apply rotation = swap X and Y size
      //if (rotated) {
      //  float aux = itemSize.X;
      //  itemSize.X = itemSize.Y;
      //  itemSize.Y = aux;
      //}
      Vector3 actualPosition = container.CalculatePositionRelativeToBottomLeftBackCorner(itemSize, itemPosition);
      BasicCuboidShape itemShape = new BasicCuboidShape(itemSize, actualPosition, index, material); 
      packingItems.Add(itemShape);
    }
  }
}